; ---------------------------------------------------------
; This program measures the time required to service IRQ1.
; ---------------------------------------------------------
    .model small,c
    .386
    .stack

    include support.asm

.data

    microSecPerCycle  dd 0.838  ; 1 / 1,193,182 Hz
    microSec          dd 0
    
.code

    ; ---------------------------------------
    ; Variables the handler needs to access.
    ; ---------------------------------------
    prevHandler       dd 0
    timerCycles       dd 0

.startup

    mov   ax, 3509h
    int   21h
    mov   WORD PTR prevHandler, bx
    mov   WORD PTR prevHandler[2], es

    push  ds
    mov   ax, 2509h
    push  cs
    pop   ds
    mov   dx, OFFSET TempHandler
    int   21h
    pop   ds

    call cls
    print  "Esc to exit, any other key for IRQ1 service time",NL
  @@:
    .IF (timerCycles)
      .IF (timerCycles == -1)
        print "overflow",NL
        mov   timerCycles, 0
      .ELSE
        fild  timerCycles
        fld   microSecPerCycle
        fmul
        fistp microSec
        invoke dw2dec,microSec,ADDR outbuf
        print ADDR outbuf," microseconds",NL
        mov   timerCycles, 0
      .ENDIF
    .ENDIF
    mov  ah, 11h
    int  16h
    jz   @B
    mov  ah, 10h
    int  16h    
    cmp  al, 27
    jne  @B

    push  ds
    lds   dx, prevHandler
    mov   ax, 2509h
    int   21h
    pop   ds

    print NL,NL,"Press any normal key to exit...",NL
    call waitkey

.exit

TempHandler:

    ; ---------------------------------------------
    ; This timing code was adapted from an example
    ; in Frank van Gilluwe's book The Undocumented
    ; PC, First Edition, Addison-Wesley, 1994,
    ; ISBN 0-201-62277-7. 
    ; ---------------------------------------------

    push  ax
    mov   al, 0b0h            ; timer 2, 16-bit, mode 0
    out   43h, al             ; send command
    IODELAY
    mov   al, 0ffh            ; write LSB of count first
    out   42h, al
    IODELAY
    out   42h, al             ; then write MSB of count

    in    al, 61h             ; read current contents
    or    al, 1               ; set gate bit on
    out   61h, al             ; activate timer
    pop   ax

    ; ----------------------------------------------
    ; Chain to the previous handler via a far call.
    ; ----------------------------------------------
    pushf                     ; duplicate action of INT
    cli                       ; duplicate action of INT
    call  dword ptr cs:prevHandler

    push  ax
    push  dx
    in    al, 61h
    test  al, 20h             ; check timer 2 output
    jz    @F                  ; jump if not overflowed

    mov   WORD PTR cs:timerCycles, -1
    mov   WORD PTR cs:timerCycles[2], -1

    pop   dx
    pop   ax
    iret

  @@:
    mov   al, 80h             ; timer 2, counter latch
    out   43h, al             ; send command
    IODELAY
    in    al, 42h             ; get lsb of counter
    IODELAY
    mov   dl, al              
    in    al, 42h             ; get msb of counter
    mov   dh, al
    mov   ax, 0ffffh          ; start count
    sub   ax, dx              ; calc elapsed cycles

    mov   WORD PTR cs:timerCycles, ax
    mov   WORD PTR cs:timerCycles[2], 0

    pop   dx
    pop   ax
    iret

end
